#ifndef __CONFIG_H__
#define __CONFIG_H__

/* Directory containing textures with trailing separator. */
#define IMGPATH "img/"

/* Directory containing sounds with trailing separator. */
#define SNDPATH "snd/"

/* Name of save file. */
#define WORLDFILE "world.sav"

/* File to save keybindings to. */
#define BINDFILE "keys.sav"

/* Size of world, the world is a cube with sides this length.
This should be a safe value. The amount of memory needed increases exponentially with this value.
Large worlds will cause performance problems because of these bottlenecks:
	lighting - worst
	scripts/phyics - next worst
	graphics memory/updaing VBOs - most difficult to fix

The amount of graphics memory used for the world will be approximately:
	size*size*size*6*8*4 bytes (lighting enabled)
	size*size*size*6*5*4 bytes (lighting disabled)

With lighting:
	For 20:  1 536 000 b =  1.536 MB
	For 40: 12 288 000 b = 12.288 MB
	For 80: 98 304 000 b = 98.304 MB

If you exceed the available graphics memory it is likely performance will be severely degraded
or the game will crash.

Estimated good values for various amounts of graphics memory - pow(bytes/2/6/8/4, 1/3):
	128MB: 69
	256MB: 87
	512MB: 110
	  1GB: 138
These values are not recommended, they cause performance problems with lighting/physics enabled! */
#define WORLDSIZE 20

/* Field of view. */
#define FOV 90

/* Multiplier for mouse movement speed. */
// NOTE: It seems SDL2 may have problems figuring out how much motion to report, especially with a trackpad
#define MOUSESPEED 0.1

/* Uncomment to disable lighting. */
// #define DISABLELIGHTING

/* Uncomment to disable fog.
Fixes problems with the sea on many GPUs. */
#define DISABLEFOG

/* Uncomment to disable phyics. */
// #define DISABLEPHYSICS

/* Uncomment to disable HUD. */
// #define DISABLEHUD

#endif
