/* World behaviour (voxel physics) functions. */

#ifndef __PHYSICS_H__
#define __PHYSICS_H__

#include "world.h"

/* Update functions for voxels. */
extern void (*physics_updatefn[VOXELTYPES])(unsigned short type, int x, int y, int z);
/* Function called when an adjacent voxel is changed by anything.
dx, dy, dz direction of the voxel that was modified */
extern void (*physics_changefn[VOXELTYPES])(unsigned short type, int x, int y, int z, char dx, char dy, char dz);
/* Function called before a voxel is destroyed by the player.
Return true to allow the voxel to be destroyed.
dx, dy, dz are the direction of the face that was selected. */
extern char (*physics_destroyfn[VOXELTYPES])(unsigned short type, int x, int y, int z, char dx, char dy, char dz);
/* Activate functions for voxels.
dx, dy, dz are the direction of the face that was selected. */
extern void (*physics_activatefn[VOXELTYPES])(unsigned short type, int x, int y, int z, char dx, char dy, char dz);
/* Click functions for tools.
selected is the type of the selected voxel or zero if no voxel is selected.
x, y, z are the coordinates of the selected voxel
dx, dy, dz are the direction of the face the player selected */
extern void (*physics_tool_destroyfn[VOXELTYPES])(unsigned short selected, int x, int y, int z, char dx, char dy, char dz);
extern void (*physics_tool_selectfn[VOXELTYPES])(unsigned short selected, int x, int y, int z, char dx, char dy, char dz);
extern void (*physics_tool_createfn[VOXELTYPES])(unsigned short selected, int x, int y, int z, char dx, char dy, char dz);

/* Clear physics functions. */
void physics_init();
/* Run a step of physics for the whole of the world. */
void physics_update();
/* Respond to a block being changed. */
void physics_changed(const int x, const int y, const int z);
/* Implementations of physics for voxels. */
// No action
void physics_tool_none(unsigned short selected, int x, int y, int z, char dx, char dy, char dz);
// Door open/close function (flood fill)
// Open replaces voxel with type+1, close replaces voxel with type-1
void physics_door_open(unsigned short type, int x, int y, int z, char dx, char dy, char dz);
void physics_door_close(unsigned short type, int x, int y, int z, char dx, char dy, char dz);
// Plants
void physics_grass(unsigned short type, int x, int y, int z);
void physics_wheat(unsigned short type, int x, int y, int z);
// Liquid
void physics_water(unsigned short type, int x, int y, int z);
void physics_lava(unsigned short type, int x, int y, int z);
// Sand
void physics_sand(unsigned short type, int x, int y, int z);

#endif
