/* Provides functions for managing player data. */

#ifndef __PLAYER_H__
#define __PLAYER_H__

#include "resource.h"
#include "world.h"

// Radius of collision box
#define PLAYERRADIUS 0.4
// Height of collision box below camera
#define PLAYERBOTTOM 1.5
// Height of collision box above camera
#define PLAYERTOP 0.4
#define VELOCITYMAX 0.1
#define GRAVITY 0.0001

/* Camera position. */
extern GLfloat cameraX, cameraY, cameraZ;
/* Pan and tilt of camera. */
extern GLfloat cameraP, cameraT;

/* Velocity of the player. */
extern float velocityX, velocityY, velocityZ;

/* Samples to play in relation to different types of voxel. */
extern Mix_Chunk *sfx[VOXELTYPES];

/* Selected voxel. */
// True if there is a valid selection
extern char selected;
// Coordinates of voxel
extern int selectX, selectY, selectZ;
// Direction of face under cursor
extern char faceX, faceY, faceZ;

/* Current tool. */
extern unsigned short player_tool;

/* Setup player.
Must be called after resource initialization. */
void player_init();
/* Clear player state. */
void player_clear();
/* Rotatate camera given relative movement of mouse. */
void player_look(const float dx, const float dy);
/* Try and move player relative to the direction they are facing.
For movement at walking pace using positive and negative one. */
void player_move(float dx, float dy, float dz);
/* Handle left click. */
void player_destroy();
/* Set tool to the type of the selected voxel. */
void player_select();
/* Create voxel (right click). */
void player_create();
/* Active a voxel. */
void player_activate();
/* Add add to current tool id if the result is valid type of voxel. */
void player_switch(const int add);
/* Toggle fly movement. */
void player_fly();
/* Physics update. */
void player_update();

#endif
