#include "voxels.h"
#include "resource.h"
#include "world.h"
#include "render.h"
#include "physics.h"
#include "player.h"

void voxels_init() {
	int texture, colour;

	// Properties
	// Opaque is used to tell the renderer which faces to remove because they are obscured
	// Solid is used to indicate whether the player can go through a voxel
	opaque[VOXEL_AIR] = 0; solid[VOXEL_AIR] = 0;
	opaque[VOXEL_TEST] = 1; solid[VOXEL_TEST] = 1;
	opaque[VOXEL_DIRT] = 1; solid[VOXEL_DIRT] = 1;
	opaque[VOXEL_GRASS] = 1; solid[VOXEL_GRASS] = 1; insideColour[VOXEL_GRASS] = 0x4BA64B;
	opaque[VOXEL_WATER] = 1; solid[VOXEL_WATER] = 0; insideColour[VOXEL_WATER] = 0x396788;
	opaque[VOXEL_LAVA] = 1; solid[VOXEL_LAVA] = 0; insideColour[VOXEL_LAVA] = 0xFF7530;
	opaque[VOXEL_PLANK] = 1; solid[VOXEL_PLANK] = 1;
	opaque[VOXEL_GLASS] = 0; solid[VOXEL_GLASS] = 1;
	opaque[VOXEL_WOOD] = 1; solid[VOXEL_WOOD] = 1;
	opaque[VOXEL_LEAF] = 1; solid[VOXEL_LEAF] = 1;
	opaque[VOXEL_STONE] = 1; solid[VOXEL_STONE] = 1; insideColour[VOXEL_STONE] = 0x838383;
	opaque[VOXEL_BLOCKS] = 1; solid[VOXEL_BLOCKS] = 1;
	opaque[VOXEL_BRICKS] = 1; solid[VOXEL_BRICKS] = 1;
	opaque[VOXEL_IRON] = 1; solid[VOXEL_IRON] = 1;
	opaque[VOXEL_BARS] = 0; solid[VOXEL_BARS] = 1;
	opaque[VOXEL_CONDUIT] = 1; solid[VOXEL_CONDUIT] = 1;
	opaque[VOXEL_PANEL] = 1; solid[VOXEL_PANEL] = 1;
	opaque[VOXEL_LARGEGRASS] = 0; solid[VOXEL_LARGEGRASS] = 0;
	opaque[VOXEL_ROSE] = 0; solid[VOXEL_ROSE] = 0;
	opaque[VOXEL_DAISY] = 0; solid[VOXEL_DAISY] = 0;
	opaque[VOXEL_SUNFLOWER] = 0; solid[VOXEL_SUNFLOWER] = 0;
	opaque[VOXEL_STEM] = 0; solid[VOXEL_STEM] = 0;
	opaque[VOXEL_MUSHROOM] = 0; solid[VOXEL_MUSHROOM] = 0;
	opaque[VOXEL_TOADSTOOL] = 0; solid[VOXEL_TOADSTOOL] = 0;
	opaque[VOXEL_SHINY] = 1; solid[VOXEL_SHINY] = 1;
	opaque[VOXEL_LAMP] = 0; solid[VOXEL_LAMP] = 0;
	opaque[VOXEL_ALARM] = 0; solid[VOXEL_ALARM] = 0;
	opaque[VOXEL_LAMPOFF] = 0; solid[VOXEL_LAMPOFF] = 0;
	opaque[VOXEL_HATCH] = 1; solid[VOXEL_HATCH] = 1;
	opaque[VOXEL_HATCHOPEN] = 0; solid[VOXEL_HATCHOPEN] = 0;
	opaque[VOXEL_FIRE] = 0; solid[VOXEL_FIRE] = 0;
	opaque[VOXEL_SCAFFOLDING] = 0; solid[VOXEL_SCAFFOLDING] = 1;
	for (colour = 0; colour < 10; colour ++) {
		opaque[VOXEL_COLOUR0+colour] = 1; solid[VOXEL_COLOUR0+colour] = 1;
	}
	for (colour = 0; colour < 4; colour ++) {
		opaque[VOXEL_GLASS0+colour] = 0; solid[VOXEL_GLASS0+colour] = 1;
	}
	for (colour = 0; colour < 5; colour ++) {
		opaque[VOXEL_LAMP0+colour] = 1; solid[VOXEL_LAMP0+colour] = 1;
	}
	opaque[VOXEL_BORNITE] = 1; solid[VOXEL_BORNITE] = 1;
	opaque[VOXEL_HAEMATITE] = 1; solid[VOXEL_HAEMATITE] = 1;
	opaque[VOXEL_GOLDNUGGET] = 1; solid[VOXEL_GOLDNUGGET] = 1;
	opaque[VOXEL_SAND] = 1; solid[VOXEL_SAND] = 1; insideColour[VOXEL_SAND] = 0xC4C2A4;
	opaque[VOXEL_BOOKSHELF] = 1; solid[VOXEL_BOOKSHELF] = 1;
	opaque[VOXEL_LARGEMUSHROOM] = 1; solid[VOXEL_LARGEMUSHROOM] = 1;
	opaque[VOXEL_TARDIS0] = 1; solid[VOXEL_TARDIS0] = 1;
	opaque[VOXEL_TARDIS1] = 1; solid[VOXEL_TARDIS1] = 1;
	opaque[VOXEL_RAINBOW] = 1; solid[VOXEL_RAINBOW] = 1;
	opaque[VOXEL_CACTUS] = 1; solid[VOXEL_CACTUS] = 1;
	opaque[VOXEL_SANDSTONE] = 1; solid[VOXEL_SANDSTONE] = 1;
	opaque[VOXEL_DEADBUSH] = 0; solid[VOXEL_DEADBUSH] = 0;
	opaque[VOXEL_CUMULUS] = 0; solid[VOXEL_CUMULUS] = 0; insideColour[VOXEL_CUMULUS] = 0xFFFFFF;
	opaque[VOXEL_NIMBUS] = 0; solid[VOXEL_NIMBUS] = 0;
	#ifdef SPP
	solid[VOXEL_CUMULUS] = 1;
	solid[VOXEL_NIMBUS] = 1;
	#endif
	for (colour = 0; colour < 4; colour ++) {
		opaque[VOXEL_WHEAT0+colour] = 0; solid[VOXEL_WHEAT0+colour] = 0;
	}
	opaque[VOXEL_LECTURN] = 1; solid[VOXEL_LECTURN] = 1;
	for (colour = 0; colour < 4; colour ++) {
		opaque[VOXEL_CABLE0+colour] = 1; solid[VOXEL_CABLE0+colour] = 1;
	}

	// Light
	// Shadow is a 24 bit RGB colour describing how much light a voxel absorbs
	//		A voxel with a 0xffffff shadow property will absorb all light
	// Glow is a 24 bit RGB colour describing how much light a voxel emits
	//		A voxel with a 0xffffff glow property will emit white light
	#ifndef DISABLELIGHTING
	shadow[VOXEL_TEST] = 0xffffff; glow[VOXEL_TEST] = 0x000000;
	shadow[VOXEL_DIRT] = 0xffffff; glow[VOXEL_DIRT] = 0x000000;
	shadow[VOXEL_GRASS] = 0xffffff; glow[VOXEL_GRASS] = 0x000000;
	shadow[VOXEL_WATER] = 0x080808; glow[VOXEL_WATER] = 0x000000;
	shadow[VOXEL_LAVA] = 0x000000; glow[VOXEL_LAVA] = 0xA48039;
	shadow[VOXEL_PLANK] = 0xffffff; glow[VOXEL_PLANK] = 0x000000;
	shadow[VOXEL_GLASS] = 0x080808; glow[VOXEL_GLASS] = 0x000000;
	shadow[VOXEL_WOOD] = 0xffffff; glow[VOXEL_WOOD] = 0x000000;
	shadow[VOXEL_LEAF] = 0xfff0ff; glow[VOXEL_LEAF] = 0x000000;
	shadow[VOXEL_STONE] = 0xffffff; glow[VOXEL_STONE] = 0x000000;
	shadow[VOXEL_BLOCKS] = 0xffffff; glow[VOXEL_BLOCKS] = 0x000000;
	shadow[VOXEL_BRICKS] = 0xffffff; glow[VOXEL_BRICKS] = 0x000000;
	shadow[VOXEL_IRON] = 0xffffff; glow[VOXEL_IRON] = 0x000000;
	shadow[VOXEL_BARS] = 0x505050; glow[VOXEL_BARS] = 0x000000;
	shadow[VOXEL_CONDUIT] = 0x000000; glow[VOXEL_CONDUIT] = 0xa0a0ff;
	shadow[VOXEL_PANEL] = 0xffffff; glow[VOXEL_PANEL] = 0x000000;
	shadow[VOXEL_LARGEGRASS] = 0x000000; glow[VOXEL_LARGEGRASS] = 0x000000;
	shadow[VOXEL_ROSE] = 0x000000; glow[VOXEL_ROSE] = 0x000000;
	shadow[VOXEL_DAISY] = 0x000000; glow[VOXEL_DAISY] = 0x000000;
	shadow[VOXEL_SUNFLOWER] = 0x000000; glow[VOXEL_SUNFLOWER] = 0x000000;
	shadow[VOXEL_STEM] = 0x000000; glow[VOXEL_STEM] = 0x000000;
	shadow[VOXEL_MUSHROOM] = 0x000000; glow[VOXEL_MUSHROOM] = 0x000000;
	shadow[VOXEL_TOADSTOOL] = 0x000000; glow[VOXEL_TOADSTOOL] = 0x000000;
	shadow[VOXEL_SHINY] = 0xffffff; glow[VOXEL_SHINY] = 0x000000;
	shadow[VOXEL_LAMP] = 0x000000; glow[VOXEL_LAMP] = 0xc0c0b0;
	shadow[VOXEL_ALARM] = 0x000000; glow[VOXEL_ALARM] = 0xc06020;
	shadow[VOXEL_LAMPOFF] = 0x000000; glow[VOXEL_LAMPOFF] = 0x000000;
	shadow[VOXEL_HATCH] = 0xffffff; glow[VOXEL_HATCH] = 0x000000;
	shadow[VOXEL_HATCHOPEN] = 0x000000; glow[VOXEL_HATCHOPEN] = 0x000000;
	shadow[VOXEL_FIRE] = 0x000000; glow[VOXEL_FIRE] = 0xf0e8e0;
	shadow[VOXEL_SCAFFOLDING] = 0x505050; glow[VOXEL_SCAFFOLDING] = 0x000000;
	for (colour = 0; colour < 10; colour ++) {
		shadow[VOXEL_COLOUR0+colour] = 0xffffff; glow[VOXEL_COLOUR0+colour] = 0x000000;
	}
	shadow[VOXEL_GLASS0] = 0x088080; glow[VOXEL_GLASS0] = 0x000000;
	shadow[VOXEL_GLASS1] = 0x080880; glow[VOXEL_GLASS1] = 0x000000;
	shadow[VOXEL_GLASS2] = 0x800880; glow[VOXEL_GLASS2] = 0x000000;
	shadow[VOXEL_GLASS3] = 0x808008; glow[VOXEL_GLASS3] = 0x000000;
	shadow[VOXEL_LAMP0] = 0x000000; glow[VOXEL_LAMP0] = 0xff6060;
	shadow[VOXEL_LAMP1] = 0x000000; glow[VOXEL_LAMP1] = 0xffff60;
	shadow[VOXEL_LAMP2] = 0x000000; glow[VOXEL_LAMP2] = 0x60ff60;
	shadow[VOXEL_LAMP3] = 0x000000; glow[VOXEL_LAMP3] = 0x6060ff;
	shadow[VOXEL_LAMP4] = 0x000000; glow[VOXEL_LAMP4] = 0xffffff;
	shadow[VOXEL_BORNITE] = 0xffffff; glow[VOXEL_BORNITE] = 0x000000;
	shadow[VOXEL_HAEMATITE] = 0xffffff; glow[VOXEL_HAEMATITE] = 0x000000;
	shadow[VOXEL_GOLDNUGGET] = 0xffffff; glow[VOXEL_GOLDNUGGET] = 0x000000;
	shadow[VOXEL_SAND] = 0xffffff; glow[VOXEL_SAND] = 0x000000;
	shadow[VOXEL_BOOKSHELF] = 0xffffff; glow[VOXEL_BOOKSHELF] = 0x000000;
	shadow[VOXEL_LARGEMUSHROOM] = 0x000000; glow[VOXEL_LARGEMUSHROOM] = 0x8080d0;
	shadow[VOXEL_TARDIS0] = 0xffffff; glow[VOXEL_TARDIS0] = 0x000000;
	shadow[VOXEL_TARDIS1] = 0xffffff; glow[VOXEL_TARDIS1] = 0x000000;
	shadow[VOXEL_RAINBOW] = 0x000000; glow[VOXEL_RAINBOW] = 0xffffff;
	shadow[VOXEL_CACTUS] = 0xffffff; glow[VOXEL_CACTUS] = 0x000000;
	shadow[VOXEL_SANDSTONE] = 0xffffff; glow[VOXEL_SANDSTONE] = 0x000000;
	shadow[VOXEL_DEADBUSH] = 0x000000; glow[VOXEL_DEADBUSH] = 0x000000;
	shadow[VOXEL_CUMULUS] = 0x505050; glow[VOXEL_CUMULUS] = 0x000000;
	shadow[VOXEL_NIMBUS] = 0x808080; glow[VOXEL_NIMBUS] = 0x000000;
	for (colour = 0; colour < 4; colour ++) {
		shadow[VOXEL_WHEAT0+colour] = 0x000000; glow[VOXEL_WHEAT0+colour] = 0x000000;
	}
	shadow[VOXEL_LECTURN] = 0xffffff; glow[VOXEL_LECTURN] = 0x000000;
	for (colour = 0; colour < 4; colour ++) {
		shadow[VOXEL_CABLE0+colour] = 0xd0d0d0; glow[VOXEL_CABLE0+colour] = 0x000000;
	}
	#endif

	// Texture maps
	// {top,bottom,left,right,front,back}Map sets the slot in the texture atlas to apply to
	//		that face of the voxel. Specifying 0 indicates that a face shouldn't be displayed
	//		texture slots can be re-used and are unlikely to be in the same order as the
	//		voxel enum
	// specialMap sets special rendering flags for a voxel
	//		RENDER_JOIN will increment the texture slot used for faces if it is surrounded
	//			by solid voxels
	//		RENDER_X will draw the voxel a cross shape, used for plants and lamps
	//			The top, left and front faces are displayed.
	texture = 0;
	topMap[VOXEL_AIR] = texture; bottomMap[VOXEL_AIR] = texture; leftMap[VOXEL_AIR] = texture; rightMap[VOXEL_AIR] = texture; frontMap[VOXEL_AIR] = texture; backMap[VOXEL_AIR] = texture; texture ++;
	topMap[VOXEL_TEST] = texture+1; bottomMap[VOXEL_TEST] = texture+1; leftMap[VOXEL_TEST] = texture; rightMap[VOXEL_TEST] = texture; frontMap[VOXEL_TEST] = texture+2; backMap[VOXEL_TEST] = texture+2; texture += 3;
	topMap[VOXEL_DIRT] = texture; bottomMap[VOXEL_DIRT] = texture+2; leftMap[VOXEL_DIRT] = texture+1; rightMap[VOXEL_DIRT] = texture+2; frontMap[VOXEL_DIRT] = texture+1; backMap[VOXEL_DIRT] = texture+2; texture += 3;
	topMap[VOXEL_GRASS] = texture+2; bottomMap[VOXEL_GRASS] = texture-1; leftMap[VOXEL_GRASS] = texture; rightMap[VOXEL_GRASS] = texture+1; frontMap[VOXEL_GRASS] = texture; backMap[VOXEL_GRASS] = texture+1; texture += 3;
	topMap[VOXEL_WATER] = texture; bottomMap[VOXEL_WATER] = texture; leftMap[VOXEL_WATER] = texture; rightMap[VOXEL_WATER] = texture; frontMap[VOXEL_WATER] = texture; backMap[VOXEL_WATER] = texture; texture ++;
	topMap[VOXEL_PLANK] = texture; bottomMap[VOXEL_PLANK] = texture+2; leftMap[VOXEL_PLANK] = texture+1; rightMap[VOXEL_PLANK] = texture+2; frontMap[VOXEL_PLANK] = texture+1; backMap[VOXEL_PLANK] = texture+2; texture += 3;
	topMap[VOXEL_GLASS] = texture; bottomMap[VOXEL_GLASS] = texture; leftMap[VOXEL_GLASS] = texture; rightMap[VOXEL_GLASS] = texture; frontMap[VOXEL_GLASS] = texture; backMap[VOXEL_GLASS] = texture; specialMap[VOXEL_GLASS] = RENDER_CONNECTED; texture += 47;
	topMap[VOXEL_WOOD] = texture+2; bottomMap[VOXEL_WOOD] = texture+3; leftMap[VOXEL_WOOD] = texture; rightMap[VOXEL_WOOD] = texture+1; frontMap[VOXEL_WOOD] = texture; backMap[VOXEL_WOOD] = texture; texture += 4;
	topMap[VOXEL_LEAF] = texture; bottomMap[VOXEL_LEAF] = texture+2; leftMap[VOXEL_LEAF] = texture+1; rightMap[VOXEL_LEAF] = texture+2; frontMap[VOXEL_LEAF] = texture+1; backMap[VOXEL_LEAF] = texture+2; texture += 3;
	topMap[VOXEL_STONE] = texture; bottomMap[VOXEL_STONE] = texture+2; leftMap[VOXEL_STONE] = texture+1; rightMap[VOXEL_STONE] = texture+2; frontMap[VOXEL_STONE] = texture+1; backMap[VOXEL_STONE] = texture+2; texture += 3;
	topMap[VOXEL_BLOCKS] = texture; bottomMap[VOXEL_BLOCKS] = texture+2; leftMap[VOXEL_BLOCKS] = texture+1; rightMap[VOXEL_BLOCKS] = texture+2; frontMap[VOXEL_BLOCKS] = texture+1; backMap[VOXEL_BLOCKS] = texture+2; texture += 3;
	topMap[VOXEL_BRICKS] = texture; bottomMap[VOXEL_BRICKS] = texture+2; leftMap[VOXEL_BRICKS] = texture+1; rightMap[VOXEL_BRICKS] = texture+2; frontMap[VOXEL_BRICKS] = texture+1; backMap[VOXEL_BRICKS] = texture+2; texture += 3;
	topMap[VOXEL_IRON] = texture; bottomMap[VOXEL_IRON] = texture+2*47; leftMap[VOXEL_IRON] = texture+2*47; rightMap[VOXEL_IRON] = texture+47; frontMap[VOXEL_IRON] = texture+2*47; backMap[VOXEL_IRON] = texture+47; specialMap[VOXEL_IRON] = RENDER_CONNECTED; texture += 3*47;
	topMap[VOXEL_BARS] = texture+1; bottomMap[VOXEL_BARS] = texture+1; leftMap[VOXEL_BARS] = texture; rightMap[VOXEL_BARS] = texture; frontMap[VOXEL_BARS] = texture; backMap[VOXEL_BARS] = texture; texture += 2;
	topMap[VOXEL_CONDUIT] = texture; bottomMap[VOXEL_CONDUIT] = texture+3; leftMap[VOXEL_CONDUIT] = texture+1; rightMap[VOXEL_CONDUIT] = texture+1; frontMap[VOXEL_CONDUIT] = texture+2; backMap[VOXEL_CONDUIT] = texture+2; texture += 4;
	topMap[VOXEL_PANEL] = texture; bottomMap[VOXEL_PANEL] = texture+3; leftMap[VOXEL_PANEL] = texture+1; rightMap[VOXEL_PANEL] = texture+1; frontMap[VOXEL_PANEL] = texture+2; backMap[VOXEL_PANEL] = texture+2; texture += 4;
	topMap[VOXEL_LARGEGRASS] = 0; bottomMap[VOXEL_LARGEGRASS] = 0; leftMap[VOXEL_LARGEGRASS] = texture; rightMap[VOXEL_LARGEGRASS] = 0; frontMap[VOXEL_LARGEGRASS] = texture; backMap[VOXEL_LARGEGRASS] = 0; specialMap[VOXEL_LARGEGRASS] = RENDER_X; texture ++;
	topMap[VOXEL_ROSE] = 0; bottomMap[VOXEL_ROSE] = 0; leftMap[VOXEL_ROSE] = texture; rightMap[VOXEL_ROSE] = 0; frontMap[VOXEL_ROSE] = texture; backMap[VOXEL_ROSE] = 0; specialMap[VOXEL_ROSE] = RENDER_X; texture ++;
	topMap[VOXEL_DAISY] = texture+1; bottomMap[VOXEL_DAISY] = 0; leftMap[VOXEL_DAISY] = texture; rightMap[VOXEL_DAISY] = 0; frontMap[VOXEL_DAISY] = texture; backMap[VOXEL_DAISY] = 0; specialMap[VOXEL_DAISY] = RENDER_X; texture += 2;
	topMap[VOXEL_SUNFLOWER] = 0; bottomMap[VOXEL_SUNFLOWER] = 0; leftMap[VOXEL_SUNFLOWER] = texture; rightMap[VOXEL_SUNFLOWER] = 0; frontMap[VOXEL_SUNFLOWER] = texture; backMap[VOXEL_SUNFLOWER] = 0; specialMap[VOXEL_SUNFLOWER] = RENDER_X; texture ++;
	topMap[VOXEL_STEM] = 0; bottomMap[VOXEL_STEM] = 0; leftMap[VOXEL_STEM] = texture; rightMap[VOXEL_STEM] = 0; frontMap[VOXEL_STEM] = texture; backMap[VOXEL_STEM] = 0; specialMap[VOXEL_STEM] = RENDER_X; texture ++;
	topMap[VOXEL_MUSHROOM] = texture+1; bottomMap[VOXEL_MUSHROOM] = 0; leftMap[VOXEL_MUSHROOM] = texture; rightMap[VOXEL_MUSHROOM] = 0; frontMap[VOXEL_MUSHROOM] = texture; backMap[VOXEL_MUSHROOM] = 0; specialMap[VOXEL_MUSHROOM] = RENDER_X; texture += 2;
	topMap[VOXEL_TOADSTOOL] = texture+1; bottomMap[VOXEL_TOADSTOOL] = 0; leftMap[VOXEL_TOADSTOOL] = texture; rightMap[VOXEL_TOADSTOOL] = 0; frontMap[VOXEL_TOADSTOOL] = texture; backMap[VOXEL_TOADSTOOL] = 0; specialMap[VOXEL_TOADSTOOL] = RENDER_X; texture += 2;
	topMap[VOXEL_SHINY] = texture; bottomMap[VOXEL_SHINY] = texture; leftMap[VOXEL_SHINY] = texture; rightMap[VOXEL_SHINY] = texture; frontMap[VOXEL_SHINY] = texture; backMap[VOXEL_SHINY] = texture; texture ++;
	topMap[VOXEL_LAMP] = 0; bottomMap[VOXEL_LAMP] = 0; leftMap[VOXEL_LAMP] = texture; rightMap[VOXEL_LAMP] = 0; frontMap[VOXEL_LAMP] = texture; backMap[VOXEL_LAMP] = 0; specialMap[VOXEL_LAMP] = RENDER_X; texture ++;
	topMap[VOXEL_ALARM] = 0; bottomMap[VOXEL_ALARM] = 0; leftMap[VOXEL_ALARM] = texture; rightMap[VOXEL_ALARM] = 0; frontMap[VOXEL_ALARM] = texture; backMap[VOXEL_ALARM] = 0; specialMap[VOXEL_ALARM] = RENDER_X; texture ++;
	topMap[VOXEL_LAMPOFF] = 0; bottomMap[VOXEL_LAMPOFF] = 0; leftMap[VOXEL_LAMPOFF] = texture; rightMap[VOXEL_LAMPOFF] = 0; frontMap[VOXEL_LAMPOFF] = texture; backMap[VOXEL_LAMPOFF] = 0; specialMap[VOXEL_LAMPOFF] = RENDER_X; texture ++;
	topMap[VOXEL_HATCH] = texture; bottomMap[VOXEL_HATCH] = texture; leftMap[VOXEL_HATCH] = texture; rightMap[VOXEL_HATCH] = texture; frontMap[VOXEL_HATCH] = texture; backMap[VOXEL_HATCH] = texture; specialMap[VOXEL_HATCH] = RENDER_CONNECTED; texture += 47;
	topMap[VOXEL_HATCHOPEN] = texture; bottomMap[VOXEL_HATCHOPEN] = texture; leftMap[VOXEL_HATCHOPEN] = texture; rightMap[VOXEL_HATCHOPEN] = texture; frontMap[VOXEL_HATCHOPEN] = texture; backMap[VOXEL_HATCHOPEN] = texture; specialMap[VOXEL_HATCHOPEN] = RENDER_CONNECTED; texture += 47;
	topMap[VOXEL_FIRE] = 0; bottomMap[VOXEL_FIRE] = 0; leftMap[VOXEL_FIRE] = texture; rightMap[VOXEL_FIRE] = 0; frontMap[VOXEL_FIRE] = texture; backMap[VOXEL_FIRE] = 0; specialMap[VOXEL_FIRE] = RENDER_X; texture ++;
	topMap[VOXEL_SCAFFOLDING] = texture; bottomMap[VOXEL_SCAFFOLDING] = texture; leftMap[VOXEL_SCAFFOLDING] = texture; rightMap[VOXEL_SCAFFOLDING] = texture; frontMap[VOXEL_SCAFFOLDING] = texture; backMap[VOXEL_SCAFFOLDING] = texture; texture ++;
	for (colour = 0; colour < 10; colour ++) {
		topMap[VOXEL_COLOUR0+colour] = texture+colour*3; bottomMap[VOXEL_COLOUR0+colour] = texture+colour*3+2; leftMap[VOXEL_COLOUR0+colour] = texture+colour*3+1; rightMap[VOXEL_COLOUR0+colour] = texture+colour*3+2; frontMap[VOXEL_COLOUR0+colour] = texture+colour*3+1; backMap[VOXEL_COLOUR0+colour] = texture+colour*3+2;
	}
	texture += colour*3;
	for (colour = 0; colour < 4; colour ++) {
		topMap[VOXEL_GLASS0+colour] = texture+colour*2; bottomMap[VOXEL_GLASS0+colour] = texture+colour*2; leftMap[VOXEL_GLASS0+colour] = texture+colour*2; rightMap[VOXEL_GLASS0+colour] = texture+colour*2; frontMap[VOXEL_GLASS0+colour] = texture+colour*2; backMap[VOXEL_GLASS0+colour] = texture+colour*2; specialMap[VOXEL_GLASS0+colour] = RENDER_JOIN;
	}
	texture += colour*2;
	for (colour = 0; colour < 5; colour ++) {
		topMap[VOXEL_LAMP0+colour] = texture+colour; bottomMap[VOXEL_LAMP0+colour] = texture+colour; leftMap[VOXEL_LAMP0+colour] = texture+colour; rightMap[VOXEL_LAMP0+colour] = texture+colour; frontMap[VOXEL_LAMP0+colour] = texture+colour; backMap[VOXEL_LAMP0+colour] = texture+colour;
	}
	texture += colour;
	topMap[VOXEL_BORNITE] = texture; bottomMap[VOXEL_BORNITE] = texture+2; leftMap[VOXEL_BORNITE] = texture+1; rightMap[VOXEL_BORNITE] = texture+2; frontMap[VOXEL_BORNITE] = texture+1; backMap[VOXEL_BORNITE] = texture+2; texture += 3;
	topMap[VOXEL_HAEMATITE] = texture; bottomMap[VOXEL_HAEMATITE] = texture+2; leftMap[VOXEL_HAEMATITE] = texture+1; rightMap[VOXEL_HAEMATITE] = texture+2; frontMap[VOXEL_HAEMATITE] = texture+1; backMap[VOXEL_HAEMATITE] = texture+2; texture += 3;
	topMap[VOXEL_GOLDNUGGET] = texture; bottomMap[VOXEL_GOLDNUGGET] = texture+2; leftMap[VOXEL_GOLDNUGGET] = texture+1; rightMap[VOXEL_GOLDNUGGET] = texture+2; frontMap[VOXEL_GOLDNUGGET] = texture+1; backMap[VOXEL_GOLDNUGGET] = texture+2; texture += 3;
	topMap[VOXEL_SAND] = texture; bottomMap[VOXEL_SAND] = texture+2; leftMap[VOXEL_SAND] = texture+1; rightMap[VOXEL_SAND] = texture+2; frontMap[VOXEL_SAND] = texture+1; backMap[VOXEL_SAND] = texture+2; texture += 3;
	topMap[VOXEL_BOOKSHELF] = texture; bottomMap[VOXEL_BOOKSHELF] = texture+3; leftMap[VOXEL_BOOKSHELF] = texture+1; rightMap[VOXEL_BOOKSHELF] = texture+2; frontMap[VOXEL_BOOKSHELF] = texture+1; backMap[VOXEL_BOOKSHELF] = texture+2; texture += 4;
	topMap[VOXEL_LARGEMUSHROOM] = texture; bottomMap[VOXEL_LARGEMUSHROOM] = texture+3; leftMap[VOXEL_LARGEMUSHROOM] = texture+1; rightMap[VOXEL_LARGEMUSHROOM] = texture+2; frontMap[VOXEL_LARGEMUSHROOM] = texture+1; backMap[VOXEL_LARGEMUSHROOM] = texture+2; texture += 4;
	topMap[VOXEL_TARDIS0] = texture+2; bottomMap[VOXEL_TARDIS0] = texture+2; leftMap[VOXEL_TARDIS0] = texture; rightMap[VOXEL_TARDIS0] = texture+1; frontMap[VOXEL_TARDIS0] = texture; backMap[VOXEL_TARDIS0] = texture+1; texture += 3;
	topMap[VOXEL_TARDIS1] = texture; bottomMap[VOXEL_TARDIS1] = texture-1; leftMap[VOXEL_TARDIS1] = texture+2; rightMap[VOXEL_TARDIS1] = texture+3; frontMap[VOXEL_TARDIS1] = texture+1; backMap[VOXEL_TARDIS1] = texture+3; texture += 4;
	topMap[VOXEL_RAINBOW] = texture; bottomMap[VOXEL_RAINBOW] = texture; leftMap[VOXEL_RAINBOW] = texture; rightMap[VOXEL_RAINBOW] = texture; frontMap[VOXEL_RAINBOW] = texture; backMap[VOXEL_RAINBOW] = texture; texture ++;
	topMap[VOXEL_CACTUS] = texture; bottomMap[VOXEL_CACTUS] = texture+2; leftMap[VOXEL_CACTUS] = texture+1; rightMap[VOXEL_CACTUS] = texture+2; frontMap[VOXEL_CACTUS] = texture+1; backMap[VOXEL_CACTUS] = texture+2; texture +=3;
	topMap[VOXEL_SANDSTONE] = texture; bottomMap[VOXEL_SANDSTONE] = texture+2; leftMap[VOXEL_SANDSTONE] = texture+1; rightMap[VOXEL_SANDSTONE] = texture+2; frontMap[VOXEL_SANDSTONE] = texture+1; backMap[VOXEL_SANDSTONE] = texture+2; texture += 3;
	topMap[VOXEL_DEADBUSH] = 0; bottomMap[VOXEL_DEADBUSH] = 0; leftMap[VOXEL_DEADBUSH] = texture; rightMap[VOXEL_DEADBUSH] = 0; frontMap[VOXEL_DEADBUSH] = texture; backMap[VOXEL_DEADBUSH] = 0; specialMap[VOXEL_DEADBUSH] = RENDER_X; texture ++;
	topMap[VOXEL_CUMULUS] = texture; bottomMap[VOXEL_CUMULUS] = texture+2; leftMap[VOXEL_CUMULUS] = texture+1; rightMap[VOXEL_CUMULUS] = texture+2; frontMap[VOXEL_CUMULUS] = texture+1; backMap[VOXEL_CUMULUS] = texture+2; texture += 3;
	topMap[VOXEL_NIMBUS] = texture; bottomMap[VOXEL_NIMBUS] = texture+2; leftMap[VOXEL_NIMBUS] = texture+1; rightMap[VOXEL_NIMBUS] = texture+2; frontMap[VOXEL_NIMBUS] = texture+1; backMap[VOXEL_NIMBUS] = texture+2; texture += 3;
	for (colour = 0; colour < 4; colour ++) {
		topMap[VOXEL_WHEAT0+colour] = 0; bottomMap[VOXEL_WHEAT0+colour] = 0; leftMap[VOXEL_WHEAT0+colour] = texture; rightMap[VOXEL_WHEAT0+colour] = 0; frontMap[VOXEL_WHEAT0+colour] = texture; backMap[VOXEL_WHEAT0+colour] = 0; specialMap[VOXEL_WHEAT0+colour] = RENDER_X; texture ++;
	}
	topMap[VOXEL_LECTURN] = texture; bottomMap[VOXEL_LECTURN] = texture+2; leftMap[VOXEL_LECTURN] = texture+1; rightMap[VOXEL_LECTURN] = texture+2; frontMap[VOXEL_LECTURN] = texture+1; backMap[VOXEL_LECTURN] = texture+2; texture += 3;
	for (colour = 0; colour < 4; colour ++) {
		topMap[VOXEL_CABLE0+colour] = texture; bottomMap[VOXEL_CABLE0+colour] = texture; leftMap[VOXEL_CABLE0+colour] = texture; rightMap[VOXEL_CABLE0+colour] = texture; frontMap[VOXEL_CABLE0+colour] = texture; backMap[VOXEL_CABLE0+colour] = texture; specialMap[VOXEL_CABLE0+colour] = RENDER_CONNECTED; texture += 47;
	}
	topMap[VOXEL_LAVA] = texture; bottomMap[VOXEL_LAVA] = texture; leftMap[VOXEL_LAVA] = texture; rightMap[VOXEL_LAVA] = texture; frontMap[VOXEL_LAVA] = texture; backMap[VOXEL_LAVA] = texture; texture ++;

	// Sfx
	// Sfx is a pointer to the sound effect to play when the voxel is create, destroyed
	//		or walked on
	sfx[VOXEL_DIRT] = sfxGrass;
	sfx[VOXEL_GRASS] = sfxGrass;
	sfx[VOXEL_PLANK] = sfxWood;
	sfx[VOXEL_GLASS] = sfxGlass;
	sfx[VOXEL_WOOD] = sfxWood;
	sfx[VOXEL_LEAF] = sfxPlant;
	sfx[VOXEL_STONE] = sfxStone;
	sfx[VOXEL_BLOCKS] = sfxStone;
	sfx[VOXEL_BRICKS] = sfxStone;
	sfx[VOXEL_IRON] = sfxMetal;
	sfx[VOXEL_BARS] = sfxMetal;
	sfx[VOXEL_CONDUIT] = sfxMetal;
	sfx[VOXEL_PANEL] = sfxMetal;
	sfx[VOXEL_LARGEGRASS] = sfxPlant;
	sfx[VOXEL_ROSE] = sfxPlant;
	sfx[VOXEL_DAISY] = sfxPlant;
	sfx[VOXEL_SUNFLOWER] = sfxPlant;
	sfx[VOXEL_STEM] = sfxPlant;
	sfx[VOXEL_MUSHROOM] = sfxPlant;
	sfx[VOXEL_TOADSTOOL] = sfxPlant;
	sfx[VOXEL_SHINY] = sfxMetal;
	sfx[VOXEL_LAMP] = sfxMetal;
	sfx[VOXEL_ALARM] = sfxMetal;
	sfx[VOXEL_LAMPOFF] = sfxMetal;
	sfx[VOXEL_HATCH] = sfxStone;
	sfx[VOXEL_HATCHOPEN] = sfxStone;
	sfx[VOXEL_SCAFFOLDING] = sfxMetal;
	for (colour = 0; colour < 10; colour ++) {
		sfx[VOXEL_COLOUR0+colour] = sfxFluff;
	}
	for (colour = 0; colour < 4; colour ++) {
		sfx[VOXEL_GLASS0+colour] = sfxGlass;
	}
	for (colour = 0; colour < 5; colour ++) {
		sfx[VOXEL_LAMP0+colour] = sfxStone;
	}
	sfx[VOXEL_BORNITE] = sfxStone;
	sfx[VOXEL_HAEMATITE] = sfxStone;
	sfx[VOXEL_GOLDNUGGET] = sfxStone;
	sfx[VOXEL_SAND] = sfxSand;
	sfx[VOXEL_BOOKSHELF] = sfxWood;
	sfx[VOXEL_LARGEMUSHROOM] = sfxPlant;
	sfx[VOXEL_TARDIS0] = sfxWood;
	sfx[VOXEL_TARDIS1] = sfxWood;
	sfx[VOXEL_CACTUS] = sfxPlant;
	sfx[VOXEL_SANDSTONE] = sfxStone;
	sfx[VOXEL_DEADBUSH] = sfxPlant;
	sfx[VOXEL_CUMULUS] = sfxFluff;
	sfx[VOXEL_NIMBUS] = sfxFluff;
	for (colour = 0; colour < 4; colour ++) {
		sfx[VOXEL_WHEAT0+colour] = sfxPlant;
	}
	sfx[VOXEL_LECTURN] = sfxWood;
	for (colour = 0; colour < 4; colour ++) {
		sfx[VOXEL_CABLE0+colour] = sfxFluff;
	}

	// Physics
	#ifndef DISABLEPHYSICS
	physics_activatefn[VOXEL_HATCH] = physics_door_open;
	physics_activatefn[VOXEL_HATCHOPEN] = physics_door_close;
	physics_updatefn[VOXEL_GRASS] = physics_grass;
	physics_updatefn[VOXEL_WHEAT0] = physics_wheat;
	physics_updatefn[VOXEL_WHEAT1] = physics_wheat;
	physics_updatefn[VOXEL_WHEAT2] = physics_wheat;
	physics_updatefn[VOXEL_WATER] = physics_water;
	physics_updatefn[VOXEL_LAVA] = physics_lava;
	physics_updatefn[VOXEL_SAND] = physics_sand;
	#endif
}
